require 'rails_helper'
require 'vcr_config'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  require_relative '../../swagger_helper'

  ACCEPTABLE_PARAMS = %w[
    first_name
    last_name
    email
    phone_number
  ]

  MUTATE_PARAMS = {
    first_name: 'String',
    last_name: 'String',
    email: 'String',
    phone_number: 'String',
  }

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ]

  HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'

  describe Api::Dmo::UsersController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:identity_provider) {create(:identity_provider, 
      key: 'als_hermes_salarie', 
      create_user_url: 'https://api-rcc.almerys.com/als-back/accounts', 
      create_user_headers: {
        'Content-Type': 'application/json',
        'X-GeXRT-API-Key': 'fbb003ae-4de4-4f0e-834f-7b509d170609',
      }
      )}
      
    let!(:entity1) {create(:entity, siret: 'Super000000001', title: 'entity1')}
    let!(:entity2) {create(:entity, siret: 'Super000000002', title: 'entity2')}
    let!(:entity3) {create(:entity, siret: 'Super000000003', title: 'entity3')}
    let!(:entity4) {create(:entity, siret: 'Super000000003', title: 'entity4')}
    let!(:admin) { create(:user, roles: ['Administrator'], group_ids: [
      entity1.manager_group_id,
      entity2.manager_group_id, 
      entity3.manager_group_id
    ]) }
    let!(:user) { create(:user, entity_rights: [{id: entity3.id}]) }

    describe 'POST /users' do
      describe 'Create as admin' do
        before :each do
          authenticated_header(request, admin)
          VCR.use_cassette("user_create_as_admin", :match_requests_on => [:method, :uri, :path]) do
            post 'create', {
                data: {
                    attributes: {
                      first_name: 'John', 
                      last_name: 'Doe', 
                      email: 'johndoe120@mail.com',
                      phone_number: '1234567890',
                      gender: 'M',
                      auth_roles: ['cre'],
                      entity_rights:[
                        {id: entity1.id.to_s, right: 'administrator'},
                        {id: entity2.id.to_s, right: 'user'},
                        {id: entity3.id.to_s, right: 'reader'},
                      ]
                    }
                }
            }
          end
        end
        it 'returns status code 201' do
          created_user = User.find_by(email: 'johndoe120@mail.com')
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['entity_rights'][0]['title']).to eq('entity1')
          expect(json['data']['attributes']['entity_rights'].length).to eq(3)
          expect(created_user.group_ids.length).to eq(3)
          expect(Hermes::Entity.find_by(id: entity1.id).writable_by?(created_user)).to eq(true)
          expect(Hermes::Entity.find_by(id: entity2.id).readable_by?(created_user)).to eq(true)
          expect(Hermes::Entity.find_by(id: entity3.id).readable_by?(created_user)).to eq(true)
        end
      end

      describe 'Error when no rights' do
        before :each do
          authenticated_header(request, user)
          VCR.use_cassette("user_create_as_no_right", :match_requests_on => [:method, :uri, :path]) do
            post 'create', {
                data: {
                    attributes: {
                      first_name: 'John', 
                      last_name: 'Doe', 
                      email: 'johndoe@mail.com',
                      phone_number: '1234567890',
                      gender: 'M',
                      auth_roles: ['cre'],
                      entity_rights:[
                        {id: entity1.id.to_s, title: entity1.title , right: 'administrator'},
                      ]
                    }
                }
            }
          end
        end
        it 'returns status code 401' do
          expect(response).to have_http_status(401)
        end
      end
    end


    describe 'PATCH /users/:id' do
      let!(:user1) { create(:user,
        first_name: 'john',
        last_name: 'doe',
        email: 'johndoe@mail.com', 
        phone_number: '1234567890',
        auth_roles: ['bailleur'],
        create_user_data: {
          gender: 'M',
          lastName: 'doe',
          firstName: 'john',
          email: 'johndoe@mail.com',
          login: 'johndoe@mail.com',
          mobileNumber: '1234567890',
          roles: ['bailleur'],
        }
      )}


      describe 'as admin' do
        before :each do
          authenticated_header(request, admin)
          VCR.use_cassette("user_patch_as_admin", :match_requests_on => [:method, :uri, :path]) do
            patch 'update', {
              id: user1.id.to_s,
              data: {
                attributes: {
                  entity_rights:[
                    {id: entity1.id.to_s, right: 'administrator'},
                    {id: entity2.id.to_s, right: 'user'},
                    {id: entity3.id.to_s, right: 'reader'},
                  ],
                  gender: 'F'
                }
              }
            }
          end
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(json['data']['attributes']['gender']).to eq('F')
          expect(response).to have_http_status(200)
          expect(json['data']['attributes']['entity_rights'].length).to eq(3)
          expect(user1.reload.group_ids.length).to eq(3)
          expect(Hermes::Entity.find_by(id: entity1.id).writable_by?(user1)).to eq(true)
          expect(Hermes::Entity.find_by(id: entity2.id).readable_by?(user1)).to eq(true)
          expect(Hermes::Entity.find_by(id: entity3.id).readable_by?(user1)).to eq(true)
          expect(Hermes::Entity.find_by(id: entity4.id).writable_by?(user1)).to eq(false)

        end
      end
    end

    describe 'GET /users/:id' do
      let!(:user1) { create(:user, email: 'johndoe@mail.com', group_ids: [entity4.manager_group_id])}

      describe 'as user' do
        before :each do
          authenticated_header(request, user1)
          get 'show', { id: user1.id.to_s  }
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(json['data']['attributes']['entity_rights'].length).to eq(1)
        end
      end
    end
  end
end
